import {
  FC,
  FormEvent,
  PropsWithChildren,
  useCallback,
  useEffect,
} from "react";
import { useObjectState } from "../../hooks/useObjectState";
import { useInput } from "../../hooks/useInput";
import { FieldEditor } from "./field-editor";
import { FieldTypes } from "./fieldtypes";

interface IProps {
  name: string;
  saveType: (arg0: string, arg1: TypeType) => void;
  type?: TypeType;
}

const constantProperties = ["metadata"];

export const TypeEditor: FC<PropsWithChildren<IProps>> = (
  { saveType, name, type: passedType },
) => {
  const {
    update: updateType,
    reset: resetType,
    state: type,
    setState: setType,
  } = useObjectState<TypeType>({});

  const {
    value: propertyName,
    bind: bindPropertyName,
    reset: resetPropertyName,
  } = useInput("", { disallowSpaces: true });

  const save = () => {
    saveType(name, type);
    resetType();
  };

  const addField = useCallback((e: FormEvent) => {
    e.preventDefault();
    updateType({
      [propertyName]: {
        type: FieldTypes.number,
        value: "",
        isConstant: false,
        limit: 1,
        minimum: 1,
      },
    });
    resetPropertyName();
  }, [propertyName, updateType, resetPropertyName]);

  const updateField = useCallback(
    (k: keyof typeof type) => (field: FieldType) => {
      updateType({ [k]: field });
    },
    [updateType],
  );

  useEffect(() => {
    passedType && setType(passedType);
  }, [passedType, setType]);

  const deleteField = useCallback((name: string) => {
    setType((t) => {
      const fields = { ...t };
      delete fields[name];
      return fields;
    });
  }, [setType]);

  return (
    <div>
      <p className="subheader">
        {passedType ? "Editing" : "Creating"} type &quot;{name}&quot;
      </p>
      <form onSubmit={addField}>
        <input type="text" {...bindPropertyName} />
        <button disabled={!propertyName}>Add Field</button>
      </form>
      <ul className="rounded-lg overflow-hidden">
        {Object.entries(type).reverse().filter(([k]) =>
          !constantProperties.includes(k)
        ).map(([key, value]) => (
          <FieldEditor
            key={"field-editor" + key}
            field={value}
            update={updateField(key)}
            fieldName={key}
            deleteField={deleteField}
          />
        ))}
      </ul>
      <div>
        <button onClick={save} disabled={!Object.keys(type).length}>
          Save Type
        </button>
      </div>
    </div>
  );
};
